// FlyingObject.h
// Base class for flying objects!
// Tony Stornetta

#ifndef __FLYING_OBJECTS__
#define __FLYING_OBJECTS__

enum DIR { NONE, LEFT, RIGHT, UP, DOWN };

// FlyingObject class
// ------------------

class FlyingObject
{
protected:
	int x_pos, y_pos;	// Object coordinates
	static int screenWidth;	// Screen width for all flying objects
	static int screenLength;// Screen length for all flying objects
public:
	int getX() { return x_pos; }
	int getY() { return y_pos; }
	static void setScreen(int w, int l) { screenWidth=w; screenLength=l; }
	virtual void move (enum DIR direction) { }
	virtual ~FlyingObject() { }
};

// SpaceShip class
// ---------------

class SpaceShip : public FlyingObject
{
protected:
	int dying;
public:
	virtual int check(int x, int y) = 0;
	void explode() { dying = 1; }
	int isDying() { return dying; }
	int at (enum DIR dir)
		{ return (dir == LEFT & x_pos == 5) || (dir == RIGHT &&
			x_pos == screenWidth - 5) || (dir == DOWN &&
			y_pos == screenLength - 3); }
};

// Player class
// ------------

class Player : public SpaceShip
{
public:
	Player (int x, int y);
	void move (enum DIR direction);
	int check(int x, int y);
	~Player();
};

// Alien class
// -----------

class Alien : public SpaceShip
{
	int cycle;
public:
	Alien (int x, int y);
	void move (enum DIR direction);
	int check(int x, int y);
	int getCycle ();
	~Alien();
};

// Bonus class
// -----------

class Bonus : public SpaceShip
{
	int cycle;
public:
	Bonus ();
	void move (enum DIR direction);
	int at (enum DIR dir)
		{ return (dir == LEFT & x_pos == 1) || (dir == RIGHT &&
			x_pos == screenWidth - 6); }	
	int check (int x, int y);
	~Bonus();
};

// Bombs class
// -----------

class Bombs : public FlyingObject
{
public:
	virtual int exists() = 0;
};

// AlienBomb class
// -----------

class AlienBomb : public Bombs
{
public:
	AlienBomb(int x, int y);
	int exists();
	void advance_on(Player *p);
	~AlienBomb();
};

// PlayerBomb class
// -----------

class PlayerBomb : public Bombs
{
public:
	PlayerBomb(int x, int y);
	int exists();
	void advance_on(Alien *s[20], Bonus *b);
	~PlayerBomb();
};

#endif // __FLYING_OBJECTS__
