// level.C
// level function
// Tony Stornetta

#include <iostream.h>
#include <stdlib.h>
#include "event.h"
#include "flyingObjects.h"
#include "level.h"

extern char getch();
extern int kbhit();
extern void stall (long);
extern void restore();

Level::Level()
{
	int i, x, y;
	char c;

	// Next level!
	time = 0;

	// Draw level welcome screen
	if (justDied)
	{
		cout << "\x1b[2J\x1b[12;" << (screenWidth - 20) / 2
			<< "HReturning to Level " << currentLevel << endl;
		cout << "\x1b[14;" << (screenWidth - 16) /2
			<< "HPress return..." << flush;

		while ((c = getch()) != '\r')
		{
        		if (c == 0x03 || c == 0x1c)
        		{
                		restore();
                		cout << "\nQuit Program" << endl;
                		exit (1);
        		}
		}
		justDied = 0;
	}
	else if (++currentLevel > 1)
	{
		if (interval > 100)
			interval -= 100;
		else
			interval = 1;
		cout << "\x1b[2J\x1b[12;" << (screenWidth - 20) / 2
			<< "HAdvancing to Level " << currentLevel << endl;
		cout << "\x1b[14;" << (screenWidth - 16) /2
			<< "HPress return..." << flush;
		while ((c = getch()) != '\r')
		{
        		if (c == 0x03 || c == 0x1c)
        		{
                		restore();
                		cout << "\nQuit Program" << endl;
                		exit (1);
        		}
		}
	}

	// Draw initial playing screen
	cout << "\x1b[2J\x1b[" << screenLength << 
		";10HLives: ";
	switch (lives)
	{
	case 1: cout << "*    "; break;
	case 2: cout << "* *  "; break;
	case 3: cout << "* * *"; break;
	}
	cout << "        Level: " << currentLevel <<
		"\x1b[" << screenLength << ";60HScore: " << score << flush;

	// Add initial events
	i = 0;
	for (x=0; x < 5; x++)
		for (y=0; y < 2; y++)
		{
			ships[i] = new Alien ((screenWidth/2)-26 + 10*x,
				3 + 5*y);
			events.add (MOVE_LEFT, time, ships[i++]);
			time++;
		}
	bonusShip = NULL;

	player = new Player ((screenWidth / 2) - 1, screenLength - 2);
	events.add (PLAYER, 5, player);
	events.add (BONUS, (rand() % 5) * 1000, NULL);
}

int Level::go()
{
	int done = 0;
	int action = 0;
	int shotsFired = 0;
	int oldscore = score;

	// Take one event at a time...
	while (!done)
	{
		Event *e = events.get();

		if (e == NULL)
		{
			cout << "Null event!" << endl;
			break;
		}

		// Update score...
		if (oldscore != score)
		{
			cout << "\x1b[" << screenLength << ";67H" <<
				score << flush;
			oldscore = score;
		}

		time = e->getTime();
		int num_dead, i;
		switch (e->getEvent())
		{
			Alien *s;
			Player *p;
			PlayerBomb *b;
			AlienBomb *a;

		case MOVE_LEFT:
			s = (Alien *)e->getObject();
			if (rand()%32 == 0)
				events.add (ABOMB, time + 10,
				   new AlienBomb (s->getX()+1, s->getY()+2));
			if (s->isDying())
				events.add (DYING, time + 20, s);
			else if ((action&0x80) && (action&0x7f)==s->getCycle())
			{
				events.add (MOVE_DOWN, time + 20, s);
				events.add (MOVE_RIGHT, time + 40, s);
			}
			else
				events.add (MOVE_LEFT, time + 20, s);
			s->move(LEFT);
			if (s->at(LEFT))
				action = s->getCycle() | 0x80;
			break;
		case MOVE_RIGHT:
			s = ((Alien *)e->getObject());
			if (s->isDying())
				events.add (DYING, time + 20, s);
			else if ((action&0x80) && (action&0x7f)==s->getCycle())
			{
				events.add (MOVE_DOWN, time + 20, s);
				events.add (MOVE_LEFT, time + 40, s);
			}
			else
				events.add (MOVE_RIGHT, time + 20, s);
			s->move (RIGHT);
			if (s->at (RIGHT))
				action = s->getCycle() | 0x80;
			break;
		case MOVE_DOWN:
			s = (Alien *)e->getObject();
			s->move (DOWN);
			if (s->at (DOWN))
				events.add (LOSE, time + 50, NULL);
			action = 0;
			break;
		case PLAYER:
			p = (Player *)e->getObject();
			if (p->isDying() > 0)
			{
				if (p->isDying() == 1)
					events.add (LOSE, time + 50, NULL);
				p->move (NONE);
			}
			else if (kbhit())
			{
				switch (getch())
				{
					case 'j':
						p->move (LEFT);
						break;
					case 'k':
						p->move (RIGHT);
						break;
					case ' ':
						// Limit player to 5 shots!
						if (++shotsFired == 6)
							shotsFired--;
						else
						   events.add (PBOMB, time + 4,
						   new PlayerBomb(p->getX()+1,
							p->getY()-1));
						break;
					case 0x03:
					case 0x1c:
						restore();
						cout << "\rQuit program!\n" <<
							endl;
						exit (1);
						break;
				}
			}
			events.add (PLAYER, time + 2, p);
			break;
		case BONUS:
			if (bonusShip == NULL)
				bonusShip = new Bonus();
			else
				bonusShip = (Bonus *)e->getObject();

			// Check for bonus ship death or disappearance
			if (bonusShip->isDying() > 1  || bonusShip->at(LEFT)
				|| bonusShip->at(RIGHT))
			{
				score += 200*bonusShip->isDying();
				delete (bonusShip);
				bonusShip = NULL;
				events.add (BONUS, time+(rand()%5)*1000, NULL);
			}
			else
			{
				events.add (BONUS, time + 30, bonusShip);
				bonusShip->move (NONE);
			}
			break;
		case PBOMB:
			b = (PlayerBomb *)e->getObject();
			b->advance_on(ships, bonusShip);
			if (b->exists())
				events.add (PBOMB, time + 4, b);
			else
			{
				shotsFired--;
				delete (b);
			}
			break;
		case ABOMB:
			a = (AlienBomb *)e->getObject();
			a->advance_on(player);
			if (a->exists())
				events.add (ABOMB, time + 10, a);
			else
				delete (a);
			break;
		case DYING:
			s = (Alien *)e->getObject();
			num_dead = 0;
			for (i=0; i<10; i++)
			{
				if (ships[i] == s)
					ships[i] = NULL;
				num_dead += (ships[i] == NULL);
			}
			if (num_dead == 10)
				events.add (WIN, time + 50, NULL);
			delete (s);
			score += 50;
			break;
		case WIN:
			done = 1;
			break;
		case LOSE:
			p = (Player *)e->getObject();
			delete (p);
			player = NULL;
			lives--;
			justDied = 1;
			done = 1;
			break;
		}
		delete (e);

		stall (interval);
	}
	return lives;
}

Level::~Level()
{
	Alien *s;
	Player *p;
	PlayerBomb *b;
	AlienBomb *a;
	Event *e;

	// Clear out event list
	while ((e = events.get()) != NULL)
	{
		switch (e->getEvent())
		{
		case ABOMB:
			a = (AlienBomb *)e->getObject();
			delete (a);
			break;
		case PBOMB:
			b = (PlayerBomb *)e->getObject();
			delete (b);
			break;
		}

		// Delete the event
		delete (e);
	}

	// Delete remaining events
	for (int i = 0; i<10; i++)
		if (ships[i] != NULL) delete (ships[i]);
	if (bonusShip != NULL)
		delete (bonusShip);
	if (player != NULL)
		delete (player);
}
